@@ -1,8 +1,8 @@
-# -*- coding: ISO-8859-1 -*-
+# -*- encoding: utf-8 -*-
 #
 #
-# Copyright (C) 2002-2006 Jrg Lehmann <joergl@users.sourceforge.net>
-# Copyright (C) 2002-2006 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2011 Jörg Lehmann <joergl@users.sourceforge.net>
+# Copyright (C) 2002-2011 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -25,54 +25,8 @@
 A canvas holds a collection of all elements and corresponding attributes to be
 displayed. """
 
-#
-# canvas item
-#
-
-from __future__ import nested_scopes
-import os
-
-class canvasitem:
-
-    """Base class for everything which can be inserted into a canvas"""
-
-    def bbox(self):
-        """return bounding box of canvasitem"""
-        pass
-
-    def processPS(self, file, writer, context, registry, bbox):
-        """process canvasitem by writing the corresponding PS code to file and
-        by updating context, registry as well as bbox
-
-        - the PS code corresponding to the canvasitem has to be written in the
-          stream file, which provides a write(string) method
-        - writer is the PSwriter used for the output
-        - context is an instance of pswriter.context which is used for keeping 
-          track of the graphics state (current linewidth, colorspace and font))
-        - registry is used for tracking resources needed by the canvasitem
-        - bbox has to be updated to include the bounding box of the canvasitem
-        """
-        raise NotImplementedError()
-
-    def processPDF(self, file, writer, context, registry, bbox):
-        """process canvasitem by writing the corresponding PDF code to file and
-        by updating context, registry as well as bbox
-
-        - the PDF code corresponding to the canvasitem has to be written in the
-          stream file, which provides a write(string) method
-        - writer is the PDFwriter used for the output, which contains properties
-          like whether streamcompression is used
-        - context is an instance of pdfwriter.context which is used for keeping
-          track of the graphics state, in particular for the emulation of PS 
-          behaviour regarding fill and stroke styles, for keeping track of the
-          currently selected font as well as of text regions.
-        - registry is used for tracking resources needed by the canvasitem
-        - bbox has to be updated to include the bounding box of the canvasitem
-        """
-        raise NotImplementedError()
-
-
-import attr, deco, deformer, document, style, trafo, type1font
+import os, tempfile
+import attr, canvasitem, deco, deformer, document, font, pycompat, style, trafo
 import bbox as bboxmodule
 
 
@@ -80,7 +34,7 @@ import bbox as bboxmodule
 # clipping class
 #
 
-class clip(canvasitem):
+class clip(canvasitem.canvasitem):
 
     """class for use in canvas constructor which clips to a path"""
 
@@ -110,15 +64,15 @@ class clip(canvasitem):
 # general canvas class
 #
 
-class _canvas(canvasitem):
+class _canvas(canvasitem.canvasitem):
 
     """a canvas holds a collection of canvasitems"""
 
-    def __init__(self, attrs=[], texrunner=None):
+    def __init__(self, attrs=None, texrunner=None):
 
         """construct a canvas
 
-        The canvas can be modfied by supplying args, which have
+        The canvas can be modfied by supplying a list of attrs, which have
         to be instances of one of the following classes:
          - trafo.trafo (leading to a global transformation of the canvas)
          - canvas.clip (clips the canvas)
@@ -135,6 +89,8 @@ class _canvas(canvasitem):
         self.items     = []
         self.trafo     = trafo.trafo()
         self.clipbbox  = None
+        if attrs is None:
+            attrs = []
         if texrunner is not None:
             self.texrunner = texrunner
         else:
@@ -145,11 +101,10 @@ class _canvas(canvasitem):
         attr.checkattrs(attrs, [trafo.trafo_pt, clip, style.strokestyle, style.fillstyle])
         # We have to reverse the trafos such that the PostScript concat operators
         # are in the right order. Correspondingly, we below multiply the current self.trafo
-        # from the right. 
-        # Note that while for the stroke and fill styles the order doesn't matter at all, 
+        # from the right.
+        # Note that while for the stroke and fill styles the order doesn't matter at all,
         # this is not true for the clip operation.
-        attrs.reverse()
-        for aattr in attrs:
+        for aattr in attrs[::-1]:
             if isinstance(aattr, trafo.trafo_pt):
                 self.trafo = self.trafo * aattr
             elif isinstance(aattr, clip):
@@ -199,24 +154,28 @@ class _canvas(canvasitem):
 
     def processPDF(self, file, writer, context, registry, bbox):
         context = context()
+        context.trafo = context.trafo * self.trafo
         if self.items:
             file.write("q\n") # gsave
             nbbox = bboxmodule.empty()
             for item in self.items:
-                if isinstance(item, type1font.text_pt):
-                    if not context.textregion:
-                        file.write("BT\n")
-                        context.textregion = 1
-                else:
-                    if context.textregion:
-                        file.write("ET\n")
-                        context.textregion = 0
-                        context.font = None
+                if isinstance(item, style.fillstyle):
+                    context.fillstyles.append(item)
+                if not writer.text_as_path:
+                    if isinstance(item, font.text_pt):
+                        if not context.textregion:
+                            file.write("BT\n")
+                            context.textregion = 1
+                    else:
+                        if context.textregion:
+                            file.write("ET\n")
+                            context.textregion = 0
+                            context.selectedfont = None
                 item.processPDF(file, writer, context, registry, nbbox)
             if context.textregion:
                 file.write("ET\n")
                 context.textregion = 0
-                context.font = None
+                context.selectedfont = None
             # update bounding bbox
             nbbox.transform(self.trafo)
             if self.clipbbox is not None:
@@ -233,8 +192,8 @@ class _canvas(canvasitem):
 
         """
 
-        if not isinstance(item, canvasitem):
-            raise RuntimeError("only instances of base.canvasitem can be inserted into a canvas")
+        if not isinstance(item, canvasitem.canvasitem):
+            raise RuntimeError("only instances of canvasitem.canvasitem can be inserted into a canvas")
 
         if attrs:
             sc = _canvas(attrs)
@@ -255,13 +214,14 @@ class _canvas(canvasitem):
         """
 
         attrs = attr.mergeattrs(attrs)
-        attr.checkattrs(attrs, [deco.deco, deformer.deformer, style.fillstyle, style.strokestyle])
+        attr.checkattrs(attrs, [deco.deco, deformer.deformer, style.fillstyle, style.strokestyle, style.fillrule])
 
         for adeformer in attr.getattrs(attrs, [deformer.deformer]):
             path = adeformer.deform(path)
 
         styles = attr.getattrs(attrs, [style.fillstyle, style.strokestyle])
-        dp = deco.decoratedpath(path, styles=styles)
+        fillrule, = attr.getattrs(attrs, [style.fillrule]) or [style.fillrule.nonzero_winding]
+        dp = deco.decoratedpath(path, styles=styles, fillrule=fillrule)
 
         # add path decorations and modify path accordingly
         for adeco in attr.getattrs(attrs, [deco.deco]):
@@ -322,8 +282,8 @@ class _canvas(canvasitem):
 #
 
 def _wrappedindocument(method):
-    def wrappedindocument(self, file, *args, **kwargs):
-        d = document.document([document.page(self, *args, **kwargs)])
+    def wrappedindocument(self, file=None, **kwargs):
+        d = document.document([document.page(self, **kwargs)])
         self.__name__ = method.__name__
         self.__doc__ = method.__doc__
         return method(d, file)
@@ -342,7 +302,7 @@ class canvas(_canvas):
     def pipeGS(self, filename="-", device=None, resolution=100,
                gscommand="gs", gsoptions="",
                textalphabits=4, graphicsalphabits=4,
-               **kwargs):
+               ciecolor=False, input="eps", **kwargs):
         if device is None:
             if filename.endswith(".png"):
                 device = "png16m"
@@ -355,6 +315,19 @@ class canvas(_canvas):
             gscommand += " -dTextAlphaBits=%i" % textalphabits
         if graphicsalphabits is not None:
             gscommand += " -dGraphicsAlphaBits=%i" % graphicsalphabits
-        gscommand += " -"
-        input = os.popen(gscommand, "w")
-        self.writeEPSfile(input, **kwargs)
+        if ciecolor:
+            gscommand += " -dUseCIEColor"
+        if input == "eps":
+            gscommand += " -"
+            pipe = pycompat.popen(gscommand, "wb")
+            self.writeEPSfile(pipe, **kwargs)
+        elif input == "pdf":
+            fd, fname = tempfile.mkstemp()
+            f = os.fdopen(fd, "wb")
+            gscommand += " %s" % fname
+            self.writePDFfile(f, **kwargs)
+            f.close()
+            os.system(gscommand)
+            os.unlink(fname)
+        else:
+            raise RuntimeError("input 'eps' or 'pdf' expected")
