@@ -1,9 +1,9 @@
-# -*- coding: ISO-8859-1 -*-
+# -*- encoding: utf-8 -*-
 #
 #
-# Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
-# Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
-# Copyright (C) 2002-2006 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2004 Jörg Lehmann <joergl@users.sourceforge.net>
+# Copyright (C) 2003-2011 Michael Schindler <m-schindler@users.sourceforge.net>
+# Copyright (C) 2002-2011 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -21,18 +21,10 @@
 # along with PyX; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-from __future__ import nested_scopes
-
 import math, warnings
 from pyx import attr, unit, text
 from pyx.graph.axis import painter, parter, positioner, rater, texter, tick
 
-try:
-    enumerate([])
-except NameError:
-    # fallback implementation for Python 2.2 and below
-    def enumerate(list):
-        return zip(xrange(len(list)), list)
 
 class _marker: pass
 
@@ -72,7 +64,7 @@ class _regularaxis(_axis):
 
     def __init__(self, min=None, max=None, reverse=0, divisor=None, title=None,
                        painter=painter.regular(), texter=texter.mixed(), linkpainter=painter.linked(),
-                       density=1, maxworse=2, manualticks=[]):
+                       density=1, maxworse=2, manualticks=[], fallbackrange=None):
         if min is not None and max is not None and min > max:
             min, max, reverse = max, min, not reverse
         self.min = min
@@ -86,6 +78,7 @@ class _regularaxis(_axis):
         self.density = density
         self.maxworse = maxworse
         self.manualticks = self.checkfraclist(manualticks)
+        self.fallbackrange = fallbackrange
 
     def createdata(self, errorname):
         return axisdata(min=self.min, max=self.max)
@@ -121,6 +114,14 @@ class _regularaxis(_axis):
         errorname = " for axis %s" % errorname
         if data.min is None or data.max is None:
             raise RuntimeError("incomplete axis range%s" % errorname)
+        if data.max == data.min:
+            if self.fallbackrange is not None:
+                try:
+                    data.min, data.max = data.min - 0.5*self.fallbackrange, data.min + 0.5*self.fallbackrange
+                except TypeError:
+                    data.min, data.max = self.fallbackrange[0], self.fallbackrange[1]
+            else:
+                raise RuntimeError("zero axis range%s" % errorname)
 
         def layout(data):
             self.adjustaxis(data, data.ticks, graphtexrunner, errorname)
@@ -154,7 +155,15 @@ class _regularaxis(_axis):
         # build a list of variants
         bestrate = None
         if self.divisor is not None:
-            partfunctions = parter.partfunctions(data.min/self.divisor, data.max/self.divisor,
+            if data.min is not None:
+                data_min_divided = data.min/self.divisor
+            else:
+                data_min_divided = None
+            if data.max is not None:
+                data_max_divided = data.max/self.divisor
+            else:
+                data_max_divided = None
+            partfunctions = parter.partfunctions(data_min_divided, data_max_divided,
                                                  self.min is None, self.max is None)
         else:
             partfunctions = parter.partfunctions(data.min, data.max,
@@ -167,15 +176,16 @@ class _regularaxis(_axis):
                 ticks = partfunction()
                 if ticks is None:
                     break
-                ticks = tick.mergeticklists(self.manualticks, ticks, mergeequal=0)
+                ticks = [t for t in tick.mergeticklists(self.manualticks, ticks, mergeequal=0)
+                         if t.ticklevel is not None or t.labellevel is not None]
                 if ticks:
                     rate = rater.rateticks(self, ticks, self.density)
                     if self.reverse:
-                        rater.raterange(self.convert(data, ticks[0]) -
-                                        self.convert(data, ticks[-1]), 1)
+                        rate += rater.raterange(self.convert(data, ticks[0]) -
+                                                self.convert(data, ticks[-1]), 1)
                     else:
-                        rater.raterange(self.convert(data, ticks[-1]) -
-                                        self.convert(data, ticks[0]), 1)
+                        rate += rater.raterange(self.convert(data, ticks[-1]) -
+                                                self.convert(data, ticks[0]), 1)
                     if bestrate is None or rate < bestrate:
                         bestrate = rate
                         worse = 0
@@ -485,7 +495,7 @@ class anchoredaxis:
         self.axis = axis
         self.errorname = errorname
         self.graphtexrunner = graphtexrunner
-        self.data = axis.createdata(errorname)
+        self.data = axis.createdata(self.errorname)
         self.canvas = None
         self.positioner = None
 
@@ -578,7 +588,7 @@ class linkedaxis(anchoredaxis):
             self.setlinkedaxis(linkedaxis)
 
     def setlinkedaxis(self, linkedaxis):
-        assert isinstance(linkedaxis, anchoredaxis), errorname
+        assert isinstance(linkedaxis, anchoredaxis), self.errorname
         self.linkedto = linkedaxis
         self.axis = linkedaxis.axis
         self.graphtexrunner = self.linkedto.graphtexrunner
